#include "tlc5947.h"

void tlc_init(void)
{
	pinMode(PLANE_1_WIRE, OUTPUT);
	pinMode(PLANE_2_WIRE, OUTPUT);
	pinMode(PLANE_3_WIRE, OUTPUT);
	pinMode(PLANE_4_WIRE, OUTPUT);
	pinMode(LATCH_WIRE, OUTPUT);
	pinMode(13, OUTPUT); // SPI Data
	pinMode(14, OUTPUT); // SPI Clock

	plane_num = 0;
	led_x = 0;
	led_y = 0;
	time_to_latch = 0;
	tlc_byte_status = 0;
}


void spi_transmit(char data)
{
	// Ansatz: Bit-Banging

	// TODO: ist chip select pin aus dem wikipedia artikel hier schon durch latch getan? oder brauche ich das noch?
	// send bits 7..0
	for (uint8_t i = 0; i < 8; i++)
	{
		// consider leftmost bit
		// set line high if bit is 1, low if bit is 0
		if (data & 0b10000000)
				digitalWrite(13, 1);
		else
				digitalWrite(13, 0);
		
		// pulse clock to indicate that bit value should be read
		digitalWrite(14, 0);
		digitalWrite(14, 1);

		// shift byte left so next bit will be leftmost
		data <<= 1;
	}

}


void latch()
{
	// Alle PLANE WIRES wieder auf 0 setzen, nachdem einer im vorherigen Durchgang im Switch auf 1 gesetzt wurde
	digitalWrite(PLANE_1_WIRE, 0);
	digitalWrite(PLANE_2_WIRE, 0);
	digitalWrite(PLANE_3_WIRE, 0);
	digitalWrite(PLANE_4_WIRE, 0);
	
	// BLANK und LATCH WIRE auf 1 setzen
	digitalWrite(BLANK_WIRE, 1);
	digitalWrite(LATCH_WIRE, 1);

	switch(plane_num)
	{
		case PLANE_1:
			digitalWrite(PLANE_1_WIRE, 1);
			plane_num = PLANE_2;
		break;
		case PLANE_2:
			digitalWrite(PLANE_2_WIRE, 1);
			plane_num = PLANE_3;
		break;
		case PLANE_3:
			digitalWrite(PLANE_3_WIRE, 1);
			plane_num = PLANE_4;
		break;
		case PLANE_4:
			digitalWrite(PLANE_4_WIRE, 1);
			plane_num = PLANE_1;
		break;
	}
	// BLANK und LATCH WIRE auf 0 setzen
	digitalWrite(LATCH_WIRE, 0);
	digitalWrite(BLANK_WIRE, 0);
}


void led_inc()
{
	switch(led_y)
	{
		case 0:
			led_x++;
			if(led_x>3)
			{
				led_x=3;
				led_y=1;
			}
		break;
		case 1:
			led_x--;
			if(led_x<0)
			{
				led_x=0;
				led_y=2;
			}
		break;
		case 2:
			led_x++;
			if(led_x>3)
			{
				led_x=3;
				led_y=3;
			}
		break;
		case 3:
			led_x--;
			if(led_x<0)
			{
				led_x=0;
				led_y=0;
				time_to_latch=1;
			}
		break;
	}
}


void transmit_tlc()
{
	uint8_t byte_to_send, tmp;

	if(time_to_latch==1)
	{
		led_x =0;
		led_y =0;
		time_to_latch=0;
		tlc_byte_status=0;	
		latch();	
	}
	else
	{
		switch(tlc_byte_status)
		{
			case 0:
				byte_to_send = ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].blue; //0bbbbBBBB // Blau wird komplett gesendet
				tlc_byte_status=1;
			break;
			case 1:
				byte_to_send = (ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].green >> 4); //0b0000gggg // Grün wird in zwei Hälften aufgeteilt und gesendet (?)
				//byte_to_send = (tmp & 0x0F); //0b00001111							 //0b0000gggg
				tlc_byte_status=2;
			break;
			case 2:
				byte_to_send = (ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].green << 4); //0bGGGG0000
				//byte_to_send = (tmp & 0xF0); //0b11110000							 //0bGGGG0000
				tlc_byte_status=3;
			break;
			case 3:
				byte_to_send = ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].red; //0brrrrRRRR // Rot wird komplett gesendet
				tlc_byte_status=4;
				led_inc();																				 // Nächste LED
			break;
			case 4:
				byte_to_send = (ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].blue >> 4);	//0b0000bbbb // Blau wird in zwei Hälften aufgeteilt und gesendet (?)
				//byte_to_send = (tmp & 0x0F); //0b00001111							 //0b0000bbbb
				tlc_byte_status=5;		
			break;
			case 5:
				byte_to_send =(ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].blue << 4);	 //0bBBBB0000
				//byte_to_send = (tmp & 0xF0); //0b11110000							 //0bBBBB0000
				tlc_byte_status=6;
			break;
			case 6:
				byte_to_send = ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].green;//0bggggGGGG // Grün wird komplett gesendet
				tlc_byte_status=7;
			break;
			case 7:
				byte_to_send = (ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].red >> 4);	 //0b0000rrrr
				//byte_to_send = (tmp & 0x0F);											//0b0000rrrr
				tlc_byte_status=8;
			break;
			case 8:
				byte_to_send =(ptr_rgb_data[led_x][switch_ary[led_y]][plane_num].red << 4);	 //0bRRRR0000
				//byte_to_send = (tmp & 0xF0);												//0bRRRR0000
				tlc_byte_status=0;
				led_inc();																			// Nächste LED
			break;
		}
		spi_transmit(byte_to_send);
	}
}
